@@ -212,54 +212,60 @@ Clip adaptors and low quality bases from Illumina reads or read pairs.
 """)
 @config.Bool_flag('adaptor_clip',
     'Do adaptor clipping?'
-)
+    )
 @config.String_flag('adaptor_file',
     'FASTA file to read adaptors from. '
     'Defaults to a built-in list of Illumina adaptors.'
-)
+    )
 @config.Int_flag('match',
     'Minimum length adaptor match.'
-)
+    )
 @config.Int_flag('max_errors',
     'Maximum errors in adaptor match.\n'
     'Note: slow for N > 1'
-)
+    )
 @config.Bool_flag('clip_ambiguous', 
     'Clip ambiguous bases, eg N'
-)
+    )
 @config.Int_flag('quality',  
     'Quality cutoff'
-)
+    )
 @config.Int_flag('qoffset',
     'Quality character offset.\n'
     'sanger: 33, solexa: 59, illumina: 64\n'
     'Will guess if not given.'
-)
+    )
 @config.Int_flag('length',
     'Reads shorter than this will be discarded.'
-)
+    )
 @config.Bool_flag('homopolymers',  
     'Set to yes to *remove* reads containing all the same base.'
-) 
+    ) 
+@config.Int_flag('trim_to',
+    'Trim ends of reads so that they are no longer than NNN bases, irrespective of quality.'
+    )
 @config.Int_flag('trim_start',
     'Trim NNN bases from start of each read irrespective of quality.'
-)
+    )
 @config.Int_flag('trim_end',                      
     'Trim NNN bases from end of each read irrespective of quality.'
-)
+    )
 @config.Bool_flag('revcom',
     'Reverse complement all reads.\n'
     'ie convert opp-out pairs to opp-in'
-)
+    )
 @config.Bool_flag('fasta',
     'Output in fasta rather than fastq format.'
-)
+    )
 @config.Bool_flag('gzip',
     'Gzip output.'
-)
+    )
 @config.Bool_flag('rejects',  
     'Output rejected reads in separate file.'
-)
+    )
+@config.Bool_flag('out_separate',  
+    'Output paired reads in separate left/right files.'
+    )
 @config.Section('reads', 'Files containing unpaired reads.')
 @config.Section('interleaved', 'Files containing interleaved read pairs.')
 @config.Grouped_section('pairs', 'Pair of files containing read pairs.') 
@@ -274,11 +280,13 @@ class Clip(config.Action_with_prefix):
     adaptor_file = None
     homopolymers = False
     revcom = False
+    trim_to = None
     trim_start = 0
     trim_end = 0
     fasta = False
     gzip = True
     rejects = False
+    out_separate = False
     
     reads = [ ]
     pairs = [ ]
@@ -298,10 +306,13 @@ class Clip(config.Action_with_prefix):
         return [ self.prefix + '_single' + self.output_suffix() ]
     
     def pairs_output_filenames(self):
-        return [ ]
+        if not self.out_separate or (not self.pairs and not self.interleaved):
+            return [ ]
+        return [ [ self.prefix + '_R1' + self.output_suffix(),
+                   self.prefix + '_R2' + self.output_suffix() ] ]
             
     def interleaved_output_filenames(self):
-        if not self.pairs and not self.interleaved:
+        if self.out_separate or (not self.pairs and not self.interleaved):
             return [ ]
         return [ self.prefix + '_paired' + self.output_suffix() ]
 
@@ -419,7 +430,8 @@ class Clip(config.Action_with_prefix):
     
         f_single = io.open_possibly_compressed_writer(self.reads_output_filenames()[0])
         if fragment_reads == 2:
-            f_paired = io.open_possibly_compressed_writer(self.interleaved_output_filenames()[0])
+            names = self.pairs_output_filenames()[0] if self.out_separate else self.interleaved_output_filenames()
+            f_paired = map(io.open_possibly_compressed_writer, names)
         if output_rejects:
             f_reject = io.open_possibly_compressed_writer(self.rejects_output_filenames()[0])
         
@@ -453,7 +465,11 @@ class Clip(config.Action_with_prefix):
             for i, (name, seq, qual) in enumerate(fragment):
                 seq = seq.upper()
                 total_in_length[i] += len(seq)
-                                    
+                
+                if self.trim_to:
+                    seq = seq[:self.trim_to]
+                    qual = qual[:self.trim_to]
+                
                 start = trim_start
                 best_start = 0
                 best_len = 0
@@ -517,22 +533,25 @@ class Clip(config.Action_with_prefix):
                     ]
             
                 if len(graduates) == 1:
-                    this_f = f_single
                     n_single += 1
+
+                    (name, seq, qual) = graduates[0]
+                    write_sequence(f_single, name, seq, qual)
                 else:
                     assert len(graduates) == 2
-                    this_f = f_paired
                     n_paired += 1
+
+                    # Write the pair to an interleaved file or separate l/r files
+                    for (lr,(name, seq, qual)) in enumerate(graduates):
+                        write_sequence(f_paired[lr%len(f_paired)], name, seq, qual)
                 
-                for name, seq, qual in graduates:
-                    write_sequence(this_f, name, seq, qual)
         
         grace.status('')
         
         if output_rejects:
             f_reject.close()
         if fragment_reads == 2:
-            f_paired.close()
+            map(lambda f: f.close(), f_paired)
         f_single.close()
         
         def summarize_clips(name, location, clips):
