@@ -120,14 +120,16 @@ then reducing the ploidy to 1 using "nesoni vcf-filter".
     'The read group should be set '
     '(nesoni\'s wrappers of read aligners do this as of version 0.87).'
     )
+@config.Int_flag('depth_limit', 'Limit depth of coverage of BAM files to this before running freebays. Use 0 for unlimited.')
 @config.Int_flag('ploidy', 'Ploidy of genotype calls.')
 @config.Float_flag('pvar', 'Probability of polymorphism.')
-@config.Section('freebayes_options', 'Flags to pass to FreeBayes.', allow_flags=True)
+@config.Section('freebayes_options', 'Flags to pass to FreeBayes.', allow_flags=True, append=False)
 class Freebayes(config.Action_with_prefix):
+    depth_limit = 100
     ploidy = 4
     pvar = 0.9
     samples = [ ]
-    freebayes_options = [ ]
+    freebayes_options = [ '--min-alternate-total', '2' ]
 
     def run(self):
         bams = [ ]
@@ -155,10 +157,20 @@ class Freebayes(config.Action_with_prefix):
             raise grace.Error('No reference FASTA file given.')
         
         with nesoni.Stage() as stage:
+            tempspace = stage.enter( workspace.tempspace() )
+            if self.depth_limit:
+                with nesoni.Stage() as stage2:
+                    for i in xrange(len(bams)):
+                        sam.Bam_depth_limit(
+                            tempspace/('%d'%i), 
+                            bams[i], 
+                            depth=self.depth_limit
+                            ).process_make(stage2)
+                        bams[i] = tempspace/('%d.bam'%i)
+            
             # FreeBayes claims to handle multiple bams, but it doesn't actually work
             if len(bams) > 1:
-                tempspace = stage.enter( workspace.tempspace() )
-                sam.Bam_merge(tempspace/'merged', bams=bams).run()
+                sam.Bam_merge(tempspace/'merged', bams=bams, index=False).run()
                 bams = [ tempspace/'merged.bam' ]
         
             command = [ 
@@ -208,6 +220,8 @@ Filtering:
 
 - variants in which genotyping failed in all samples are removed
 
+- variants in which all genotypes match the reference are removed
+
 Note:
 
 - phased VCF files are not supported
@@ -250,6 +264,8 @@ class Vcf_filter(config.Action_with_prefix):
     def _blank_gt(self):
         return '/'.join(['.']*self.ploidy)
     
+    def _reference_gt(self):
+        return '/'.join(['0']*self.ploidy)
 
     def _make_sample_dirichlet(self, variants, sample):
         counts = get_variant_counts(sample)
@@ -317,7 +333,12 @@ class Vcf_filter(config.Action_with_prefix):
         #print 
         #print reader.infos
         #print 
+        
+        n = 0
+        n_kept = 0
+        
         for record in reader:
+            n += 1
             variants = get_variants(record)
             
             any = False
@@ -325,7 +346,7 @@ class Vcf_filter(config.Action_with_prefix):
             for sample in record.samples:
                 self._modify_sample(variants, sample)
                 
-                any = any or sample.data.GT != self._blank_gt()
+                any = any or (sample.data.GT != self._blank_gt() and sample.data.GT != self._reference_gt())
 
                 #print call.sample
                 #for key in call.data._fields:
@@ -350,9 +371,14 @@ class Vcf_filter(config.Action_with_prefix):
             
             if any:
                 writer.write_record(record)
+                n_kept += 1
                 
         writer.close()
         reader_f.close()
+        
+        self.log.datum('variants','input', n)
+        self.log.datum('variants','kept',  n_kept)
+        
         index_vcf(self.prefix+'.vcf')
 
 
@@ -375,6 +401,7 @@ class Snpeff(config.Action_with_prefix):
                 JAR=jar, GENOME=reference.name, VCF=self.vcf, CONFIG=reference/'snpeff.config',
                 stdout=f)
 
+        index_vcf(self.prefix+'.vcf')
 
 
 
